import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as ss
import pystan
import seaborn as sb

flnm = input('Data File Name (*.xlsx) = ')
xlsx = pd.ExcelFile(flnm)                   #   入力ファイル名の設定
data = pd.read_excel(xlsx)                  #   読み込み

VarLabels = data.columns[1:].values;
print(VarLabels)

X = data.values[:, 1:]
print(X[:10])
N = len(X)
M = len(X[0])
print('N = ', N)
print('M = ', M)

Data = {'N': N, 'M': M, 'X': X}

def f_init():
    return dict(mu = [0.0]*M, psy = 1.0, F = [0.0]*N)

sm = pystan.StanModel(file = 'Reli_2Cat.stan')
fit = sm.sampling(data = Data, pars = ['mu', 'psy', 'F'], init = f_init, n_jobs = 1)

print(fit)


Mus = fit['mu']
for i in range(M):
    sb.kdeplot(Mus.T[i])
plt.title('$\mu$', fontsize = 16)
plt.show()
print()
for j in range(M):
    print('mu[{0:}] = {1:.5}'.format(j+1, np.percentile(Mus.T[j], 50)))

Psys = fit['psy']
rho = []
for i in range(len(Psys)):
    rho.append(M**2 / (M**2 + M*(Psys[i]**2)))
rho_Q1, rho_Med, rho_Q3 = np.percentile(rho, [25, 50, 75])
rho_Mean = np.array(rho).mean()

print('\nomega(mean) = {0:.5}'.format(rho_Mean))
print('omega(Med.) = {0:.5}'.format(rho_Med))
print('omega(Q1) = {0:.5}     omega(Q3) = {1:.5}'.format(rho_Q1, rho_Q3))

sb.kdeplot(rho)
plt.title('Mean = {0:.3}\nQ1 = {1:.3}, Med.= {2:.3}, Q3 = {3:.3}'.
          format(rho_Mean, rho_Q1, rho_Med, rho_Q3), fontsize = 14)
plt.xlabel('$\omega$', fontsize = 16)
plt.show()

Xtot = X.sum(axis = 1)
Fs = fit['F']
Rs = []
R2s = []
for t in range(len(Fs)):
    r = np.corrcoef(Xtot, Fs[t])[0][1]
    Rs.append(r)
    R2s.append(r**2)
R_Q1, R_med, R_Q3 = np.percentile(Rs, [25, 50, 75])
R2_Q1, R2_med, R2_Q3 = np.percentile(R2s, [25, 50, 75])
sb.kdeplot(Rs, label = 'R')
sb.kdeplot(R2s, label = '$R^2$')
plt.title('R(Med.) = {0:.3}   $R^2$(Med.) = {1:.3}'.format(R_med, R2_med), fontsize = 16)
plt.show()

print('\nR^2(Med.) = {0:.5}'.format(R2_med))
print('R^2(Q1) = {0:.5}     R^2(Q3) = {1:.5}'.format(R2_Q1, R2_Q3))
print('R(Med.) = {0:.5}'.format(R_med))
print('R(Q1) = {0:.5}     R(Q3) = {1:.5}'.format(R_Q1, R_Q3))

S = np.cov(X, rowvar = False)

VarX = np.var(X.sum(axis = 1))

alpha_coef = (M/(M-1)) * (1 - (np.diag(S).sum() / VarX))
print('\nalpha = {0:.5}'.format(alpha_coef))

Cor = np.corrcoef(X, rowvar = False)

r_bar = (Cor.sum() - np.diag(Cor).sum()) / (M * (M - 1))

std_alpha = M * r_bar / (1.0 + (M - 1) * r_bar)
print('standardized alpha = {0:.5}'.format(std_alpha))

